import requests
import traceback
import json
import os
import sys
from datetime import datetime
import base64


class PanelConnctor:
    def __init__(self, link, port, password="6y2oIyGo2CA8*Su!#vY") -> None:
        self.password = password
        if link.endswith('/'):
            link = link[:-1]
        self.url = f"{link}:{port}/panel"

    def request(self,path,data):
        res=None
        res=requests.post(f"{self.url}/{path}",data=json.dumps(data),timeout=3).json()
        return res
    
    def make_service(self, remark: str, volume: float, ex_day: int):
        try:
            cfg = {
                "remark": remark,
                "volume": volume,
                "expire": ex_day,
                "password": self.password
            }
            res = self.request('create',cfg)
            return res
        except Exception as e:
            debug(e)
        return {}

    def rebuy_service(self, remark: str, volume: float, ex_day: int,name:str):
        try:
            cfg = {
                "remark": remark,
                "volume": volume,
                "expire": ex_day,
                'name':name,
                "password": self.password
            }
            res = self.request('rebuy',cfg)
            return res
        except Exception as e:
            debug(e)
        return {}

    def make_batch_service(self, remark: str, volume: float, ex_day: int,batch:int):
        try:
            cfg = {
                'batch':batch,
                "remark": remark,
                "volume": volume,
                "expire": ex_day,
                "password": self.password
            }
            res = self.request('create_batch',cfg)
            return res
        except Exception as e:
            debug(e)
        return {}

    def get_inbound_by_clients(self, remark: str,name:str=None):
        try:
            cfg = {
                "remark": remark,
                "name":name,
                "password": self.password
            }
            res = self.request('accounts',cfg)
            
            return res
        except Exception as e:
            debug(e)
        return {}

    def delete_clients(self, name: str, remark: str):
        try:
            cfg = {
                "name": name,
                "remark": remark,
                "password": self.password
            }
            res = self.request('delete_account',cfg)
            return res
        except Exception as e:
            debug(e)
        return {}

    def add_day_to_all_config(self, remark: str,extra_day:int):
        try:
            cfg = {
                "remark": remark,
                'extra_day':extra_day,
                "password": self.password
            }
            res = self.request('add_time_all_clients',cfg)
            return res
        except Exception as e:
            debug(e)
        return {}

    def add_day_to_single_config(self, remark: str,name:str,extra_day:int):
        try:
            cfg = {
                "remark": remark,
                'name':name,
                'extra_day':extra_day,
                "password": self.password
            }
            res = self.request('add_time',cfg)
            return res
        except Exception as e:
            debug(e)
        return {}

    def change_accounts(self, remark: str,names:list,extra_day:int,extra_volume:int):
        try:
            cfg = {
                "remark": remark,
                'names':names,
                'extra_volume':extra_volume,
                'extra_day':extra_day,
                "password": self.password
            }
            res = self.request('change_accounts',cfg)
            return res
        except Exception as e:
            debug(e)
        return {}

    
    @staticmethod
    def sep_clients_info(info):
        try:
            clients = info['clients']
            info = {k: v for k, v in info.items() if k != 'clients'}
            clients = {i['name']: i for i in clients if 'name' in i}
            return clients, info
        except:
            pass
        return {}, {}

    @staticmethod
    def config_link_maker(name,info,alternative_link,protocol,network,security,tag='',alter_host='',alter_sni='',alter_port=0):
        try:
            
            clients, info = PanelConnctor.sep_clients_info(info)
            
            client = clients.get(name)
            id = client.get('id')
            base_link = info.get('remark')
            
            if info.get('panel')=='hiddify':
                host = base_link if alter_host=='' else alter_host
                base_link = alternative_link if alternative_link!='' else base_link
                if security in ['reality','tls']:
                    port = int(str(info.get(f'tls_ports')).split(',')[0])
                else:port = int(str(info.get(f'{security}_ports')).split(',')[0])
                
                path = "/"+info.get(f'path_{protocol}')+info.get(f'path_{network}')
            else:
                if alternative_link!='':
                    base_link= alternative_link
                path = info.get('wsSettings',{}).get('path','')
                port = info.get('port',0)
                network = info.get('network','')
                security = info.get('security','')
                serverName = info.get('tlsSettings',{}).get('serverName','')
                if serverName!='':base_link=serverName
                if alter_host=='':
                    host =  info.get('headers',{}).get('Host','')
                    if network=='tcp':
                        thost =  info.get('tcpSettings',{}).get('header',{}).get('request',{}).get('headers',{}).get('Host','')
                        if thost:host=thost[0]
                else:host = alter_host
                
            sni = alter_sni if alter_sni!='' else host
            tls='tls' if security=='tls' else 'none'
            if alter_port!=0:port = alter_port
            if network == 'ws':
                if protocol=='vmess':
                    out = {
                        "v": "2",
                        "ps": name+tag,
                        "add": base_link,
                        "port": port,
                        "id": id,
                        "aid": 0,
                        "net": network,
                        "type": "none",
                        "host": host,
                        "path": path,
                        "tls": tls
                    }
                    out = json.dumps(out)
                    out = base64.b64encode(out.encode()).decode()
                    out = f"{protocol}://{out}"
                    return out
                elif protocol=='vless' or protocol=='trojan':
                    out = f"{protocol}://{id}@{base_link}:{port}?type={network}&security={tls}&alpn=http/1.1"
                    out += f"&path={path}&host={host}&fp=chrome&headerType=none"
                    if protocol=='vless':
                        out += '&encryption=none'
                    if security=='tls': out+=f"&sni={sni}"
                    out+=f"#{name}{tag}"
                    return out
            elif network == 'grpc':
                out=f"{protocol}://{id}@{base_link}:{port}?"
                if protocol in['trojan','vless']:
                    service_name = path[1:]
                    if security=='reality':
                        reality_fallback_domain = info.get('reality_fallback_domain')
                        pbk = info.get('reality_public_key')
                        sid = info.get('reality_short_ids')
                        out+=f"hiddify=1&sni={reality_fallback_domain}&type={network}&alpn=h2"
                        out+=f"&path={service_name}&serviceName={service_name}&mode=multi&encryption=none&fp=chrome"
                        out+=f"&headerType=None&security=reality&pbk={pbk}&sid={sid}"
                        out+=f"#{name}{tag}"
                        return out
                    else:
                        out+=f"security={tls}&type={network}&alpn=http/1.1"
                        out += f"&path={path}&host={host}&serviceName={service_name}&fp=chrome&headerType=none&mode=multi"
                        if protocol=='vless':
                            out += '&encryption=none'
                        if security=='tls': out+=f"&sni={sni}"
                        out+=f"#{name}{tag}"
                        return out
            elif network == 'tcp':
                if security=='reality':
                    reality_fallback_domain = info.get('reality_fallback_domain')
                    pbk = info.get('reality_public_key')
                    sid = info.get('reality_short_ids')
                    if protocol in['trojan','vless']:
                        out=f"{protocol}://{id}@{base_link}:{port}?"
                        out+=f"security=reality&alpn=h2"
                        out+=f"&headerType=none&fp=chrome&type=tcp&flow=xtls-rprx-vision"
                        if protocol=='vless':
                            out += '&encryption=none'
                        out+=f"&pbk={pbk}"
                        out+=f"&sid={sid}"
                        out+=f"&sni={sni}"
                        out+=f"#{name}{tag}"
                        return out
                if info.get('panel')=='xui':
                    if protocol in['trojan','vless']:
                        service_name = path[1:]
                        out=f"{protocol}://{id}@{base_link}:{port}?"
                        out += f"security=none&type={network}"
                        out += f"&path={path}&host={host}&headerType=http"
                        if protocol=='vless':
                            out += '&encryption=none'
                        out+=f"#{name}{tag}"
                        return out
                    
            
        except Exception as e:
            debug(e)
        return ''


def debug(error, debug_mode=True):
    exc_type, exc_obj, exc_tb = sys.exc_info()
    fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
    pm = datetime.now().strftime("%H:%M") + \
        f" , {fname}:{exc_tb.tb_lineno} => "
    try:
        if not isinstance(error, str):
            err = traceback.format_exc()
            errl = err.split('\n')
            err_msg = '\n\t'.join(errl[1:])
            pm += '\n'+err_msg
        else:
            error = str(error)
            pm += '\t'+error
        PATH = os.path.abspath(os.path.join(
            os.path.dirname(__file__), '../..'))
        with open(PATH+'/log', 'a+') as f:
            f.write(pm+'\n\n')
            if debug_mode:
                print(pm)

    except:
        pass



def chunks(lst, n, reverse=False):
    """return successive n-sized chunks from lst."""
    res = []
    for i in range(0, len(lst), n):
        if reverse:
            res += [reversed(lst[i:i + n])]
        else:
            res += [lst[i:i + n]]
    return res


def chunk_dict(dict_in,lim):
    out =[]
    temp={}
    x=0
    for k,v in dict_in.items():
        if x!=0 and x%lim==0:
            out+=[temp]
            temp={}
        x+=1
        temp[k]=v
    if temp:out+=[temp]
    return out



import CloudFlare
class CloudFlareHandler:
    def __init__(self,email,api) -> None:
        self.cf = CloudFlare.CloudFlare(email,token=api)
    def get_dns_records(self,domain):
        params = {'name': domain}
        zones = self.cf.zones.get(params=params)
        zone = [i for i in zones if i['name']==domain]
        if not zone:
            raise Exception('domain not exists')
        zone_id = zone[0]['id']
        dns_records = self.cf.zones.dns_records.get(zone_id)
        return dns_records
        
    def update_dns_records(self,domain,sub,record_type,new_content=None,new_type=None,proxy:bool=None):
        record_type = record_type.upper()
        if new_type:new_type.upper()
        params = {'name': domain}
        zones = self.cf.zones.get(params=params)
        zone = [i for i in zones if i['name']==domain]
        if not zone:
            raise Exception('domain not exists')
        zone_id = zone[0]['id']
        if '.' not in sub:sub=sub+f".{domain}"
        
        dns_records = self.cf.zones.dns_records.get(zone_id)
        dns_records = [i for i in dns_records if i['name']==sub and i['type']==record_type]
        if not dns_records:
            raise Exception('dns_records not exists')
        record = dns_records[0]
        if not new_content:new_content=record['content']
        if not new_type:new_type=record['type']
        
        data={'name':record['name'],'content':new_content,'type':new_type,}
        if proxy!=None:
            data['proxied']=proxy
        
        self.cf.zones.dns_records.put(zone_id,record['id'],data=data)
        if self.purge_cache(zone_id):
            return True
        raise Exception('cache dont be purged')
    
    def purge_cache(self,zone_id=None,domain=None):
        if not zone_id and not domain:
            raise Exception('fill one of inputs parameters')
        if not zone_id:
            params = {'name': domain}
            zones = self.cf.zones.get(params=params)
            zone = [i for i in zones if i['name']==domain]
            if not zone:
                raise Exception('domain not exists')
            zone_id = zone[0]['id']
        
        self.cf.zones.purge_cache.post(zone_id,data={'purge_everything':True})
        return True
        
    def delete_dns_records(self,domain,sub,record_type):
        record_type = record_type.upper()
        params = {'name': domain}
        zones = self.cf.zones.get(params=params)
        zone = [i for i in zones if i['name']==domain]
        if not zone:
            raise Exception('domain not exists')
        zone_id = zone[0]['id']
        if '.' not in sub:sub=sub+f".{domain}"
        
        dns_records = self.cf.zones.dns_records.get(zone_id)
        dns_records = [i for i in dns_records if i['name']==sub and i['type']==record_type]
        if not dns_records:
            raise Exception('dns_records not exists')
        record = dns_records[0]
        
        self.cf.zones.dns_records.delete(zone_id,record['id'])
        if self.purge_cache(zone_id):
            return True
        raise Exception('cache dont be purged')
        
    def add_dns_records(self,domain,sub,record_type,content,proxy:bool=None):
        record_type = record_type.upper()
        params = {'name': domain}
        zones = self.cf.zones.get(params=params)
        zone = [i for i in zones if i['name']==domain]
        if not zone:
            raise Exception('domain not exists')
        zone_id = zone[0]['id']
        if '.' not in sub:sub=sub+f".{domain}"
        
        
        
        data={'name':sub,'content':content,'type':record_type}
        if proxy!=None:
            data['proxied']=proxy
        
        self.cf.zones.dns_records.post(zone_id,data=data)
        return True
    
    def get_all_domain(self):
        cf = self.cf
        zones = cf.zones.get()
        return [i['name'] for i in zones]
if __name__ == "__main__":
    api = '190ffb1842cf0fa55886cc0622803ed17ef0c'
    # token= 'JEWrIf5xnQP3SXlFqIIt8MiCwY80Q3dN-2EALo1p'
    # cert='v1.0-b954a062aef284635f66aab1-94d0ebae5d55c9f0a326ebdd89b1b5ec9e124df22bce16a9cc10d2180c144a2f6bb09489f1dc81075382d696f4e208065574a7c12892cb6de54d32cec0d24c317626ff349c2bd14a89'
    domain='smartrad.shop'
    cf = CloudFlareHandler('m.tahmasbi0111@yahoo.com',api=api)
    # o= [(i['name'],i['type']) for i in cf.get_dns_records(domain)]
    # cf.add_dns_records(domain,'foo','cname','ip1.smartrad.shop')
    o=cf.get_all_domain()
    print(o)
    # cf.update_dns(domain,'foo','cname','ip1.smartrad.shop',proxy=True)
    # cf.purge_cache(domain)
    


from lxml import etree
from bs4 import BeautifulSoup
import requests


class Scraper:
    def __init__(self, url=None,root=None,proxy=None) -> None:
        '''
            proxy = {'https':'http://127.0.0.1:8889'}
        '''
        self.url = url
        self.proxy = proxy
        if url:
            self.root = self.soup()
        elif root:
            self.root =root
        else:
            raise Exception('root not defind')

    def soup(self):
        try:
            HEADERS = ({'User-Agent':
                        'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 \
                        (KHTML, like Gecko) Chrome/44.0.2403.157 Safari/537.36',
                        'Accept-Language': 'en-US, en;q=0.5'})

            webpage = requests.get(self.url, headers=HEADERS,proxies=self.proxy)
            soup = BeautifulSoup(webpage.content, "html.parser")
            
            return soup
        except:print('Error Request To Url')

    def find_all(self, tag, attrs={}):
        '''
            attrs = {'attribut name':'attribut value'}
            ex: find_all( 'div', attrs'= {'class': 'teaser3})
        '''
        if attrs:
            attrs={'attrs':attrs}
        return self.root.find_all(tag, **attrs)

    def find(self,  tag, attrs={}):
        '''
            attrs = {'attribut name':'attribut value'}
            ex: find_all( 'div', attrs'= {'class': 'teaser3})
        '''
        if attrs:
            attrs = {'attrs':attrs}
        return self.root.find(tag, **attrs)

    def xpath(self, xpath:str):
        dom = etree.HTML(str(self.root))
        return dom.xpath(xpath)


from datetime import datetime,timedelta
import pytz


def get_timestamp(str_input='',sec=None,min=None,hour=None,day=None,month=None,m_sec=None,ms = True,tehran=True):
    ''' 
        
        if inp > 0 replace with inp else do decrease
        inputs = month , day , sec
        
        
        str_input:now , today , yesterday,this_month,last_month
        return ts
    '''
    
    if tehran:
        tz = pytz.timezone('Asia/Tehran')
        now = datetime.now(tz=tz)
    else:now = (datetime.now())

    if str_input=='today':
        sec=0
        min=0
        hour=0
    elif str_input=='yesterday':
        sec=0
        min=0
        hour=0
        day=-1
    
    elif str_input=='this_month':
        sec=0
        min=0
        hour=0
        day=1
    elif str_input=='last_month':
        sec=0
        min=0
        hour=0
        day=1
        month=now.month-1
    
    if m_sec!=None:
        if m_sec<0:
            now = now-timedelta(milliseconds=abs(m_sec))
        else:
            now = now.replace(microsecond=m_sec)

    if sec!=None:
        if sec<0:
            now = now-timedelta(seconds=abs(sec))
        else:
            now = now.replace(second = sec)
    
    if min!=None:
        if min<0:
            now = now-timedelta(minutes=abs(min))
        else:
            now = now.replace(minute=min)
            
    if hour!=None:
        if hour<0:
            now = now-timedelta(hours=abs(hour))
        else:
            now = now.replace(hour = hour)
    
    if day!=None:
        if day<0:
            now = now-timedelta(days=abs(day))
        else:
            now = now.replace(day = day)
       
    if month!=None:
        if month<0:
            now = now-timedelta(months=abs(month))
        else:
            now = now.replace(month = month)
    
        
    res = now.timestamp()
    if ms:
        res*=1000
    return int(res)

def convert_to_ms(ts):
    if len(str(ts))<13:
        return ts*1000
    return ts



from setuptools import setup,find_packages
import os,glob
requires = [
    'cffi==1.15.1',
    'cryptography==41.0.3',
    'pycparser==2.21',
    'beautifulsoup4==4.12.2',
    'lxml==4.9.3',
    'requests==2.31.0',
    'easy-db==0.9.15',
    'hcloud==1.26.0',
    'cloudflare==2.11.6',
    'python-socks==2.4.3'


]

setup(
    name = 'Starco',
    version='2.4.1',
    author='Mojtaba Tahmasbi',
    packages=find_packages(),
    install_requires=requires,
)